@@ -1,22 +1,12 @@
-from abc import abstractmethod
 from collections.abc import Callable
 from functools import wraps
 from inspect import iscoroutinefunction
-from typing import Any, Protocol, runtime_checkable
+from typing import Any
 
 from injection._core.common.asynchronous import Caller
 from injection._core.module import Module, mod
 
-type AsFunctionWrappedType[**P, T] = type[AsFunctionCallable[P, T]]
-
-
-@runtime_checkable
-class AsFunctionCallable[**P, T](Protocol):
-    __slots__ = ()
-
-    @abstractmethod
-    def call(self, *args: P.args, **kwargs: P.kwargs) -> T:
-        raise NotImplementedError
+type AsFunctionWrappedType[**P, T] = type[Callable[P, T]]
 
 
 def asfunction[**P, T](
@@ -29,8 +19,8 @@ def asfunction[**P, T](
     module = module or mod()
 
     def decorator(wp: AsFunctionWrappedType[P, T]) -> Callable[P, T]:
-        fake_method = wp.call.__get__(NotImplemented)
-        factory: Caller[..., AsFunctionCallable[P, T]] = module.make_injected_function(
+        fake_method = wp.__call__.__get__(NotImplemented, wp)
+        factory: Caller[..., Callable[P, T]] = module.make_injected_function(
             wp,
             threadsafe=threadsafe,
         ).__inject_metadata__
@@ -42,14 +32,14 @@ def asfunction[**P, T](
             @wraps(fake_method)
             async def wrapper(*args: P.args, **kwargs: P.kwargs) -> Any:
                 self = await factory.acall()
-                return await self.call(*args, **kwargs)  # type: ignore[misc]
+                return await self(*args, **kwargs)  # type: ignore[misc]
 
         else:
 
             @wraps(fake_method)
             def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
                 self = factory.call()
-                return self.call(*args, **kwargs)
+                return self(*args, **kwargs)
 
         wrapper.__name__ = wp.__name__
         wrapper.__qualname__ = wp.__qualname__
